Scriptname Hardcore:HC_ManagerScript extends Quest

;**************************************************************************************************
;**************************************   INITIALIZATION    ***************************************
;**************************************************************************************************

Event OnInit()
	PlayerRef = Game.GetPlayer()

	RegisterForRemoteEvent(PlayerRef, "OnKill") ;for awarding Adrenaline

	RegisterForPlayerSleep() ; for resetting adrenaline

	RegisterForMenuOpenCloseEvent("PipboyMenu") ;convenient hook to turn toggle fast travel

	RegisterForRemoteEvent(PlayerRef, "OnCombatStateChanged") ;for hunger effects
	RegisterForRemoteEvent(PlayerRef, "OnItemEquipped") ;for hunger effects

	StartTimerGameTime(GameTimerInterval_SleepDeprivation, GameTimerID_SleepDeprivation) 
	StartTimerGameTime(GameTimerInterval_Encumbrance, GameTimerID_Encumbrance) 
	StartTimerGameTime(GameTimerInterval_Sustenance, GameTimerID_Sustenance)
	StartTimerGameTime(GameTimerInterval_Disease, GameTimerID_Disease)

	HandleEncumbranceTimer()
	
	trace(self, "OnInit() finished.")
EndEvent



;**************************************************************************************************
;**************************************       COMMON        ***************************************
;**************************************************************************************************

Group CommonProperties
	Formlist Property HC_Rules const auto
EndGroup

Actor PlayerRef ;pointwe to player, set in OnInit()

;Global "enum" values, used by IsGlobalTrue, SetGlobal
int iGlobalTrue = 1 const
int iGlobalFalse = 0 const

;************************************     TIMERS    *********************************************
int GameTimerID_SleepDeprivation = 1 const
int GameTimerID_Encumbrance = 2 const
int GameTimerID_Sustenance = 3 const
int GameTimerID_Disease = 4 const

Group Timers

	float Property GameTimerInterval_SleepDeprivation = 24.0 auto const ;hours

	float Property GameTimerInterval_Encumbrance = 24.0 auto const ;hours

	float Property GameTimerInterval_Sustenance = 0.1 auto const ;hours -- THIS NEEDS TO BE LESS THAN TickHoursCostPerCombat
{THIIS NEEDS TO BE LESS THAN TickHoursCostPerCombat }

	float Property GameTimerInterval_Disease = 12.0 auto const ;hours

EndGroup


Event OnTimerGameTime(int aiTimerID)
	trace(self, "OnTimerGameTime() aiTimerID: " + aiTimerID )

	if aiTimerID == GameTimerID_SleepDeprivation
		trace(self, "OnTimerGameTime() aiTimerID: = GameTimerID_SleepDeprivation" )
	
		if IsGlobalTrue(HC_Rule_SleepEffects) && ProcessingSleep == false
			HandleSleepDeprivationTimer()
		endif
		;this needs to live outside the if check, in case we allow turning on option during play    
		StartTimerGameTime(GameTimerInterval_SleepDeprivation, GameTimerID_SleepDeprivation)

	elseif aiTimerID == GameTimerID_Encumbrance
		trace(self, "OnTimerGameTime() aiTimerID: = GameTimerID_Encumbrance" )
		if IsGlobalTrue(HC_Rule_DamageWhenEncumbered)
			HandleEncumbranceTimer()
		endif
		;this needs to live outside the if check, in case we allow turning on option during play
		StartTimerGameTime(GameTimerInterval_Encumbrance, GameTimerID_Encumbrance)

	elseif aiTimerID == GameTimerID_Sustenance
		trace(self, "OnTimerGameTime() aiTimerID: = GameTimerID_Sustenance" )
		if IsGlobalTrue(HC_Rule_SustenanceEffects)
			HandleSustenanceTimer()
		endif
		;this needs to live outside the if check, in case we allow turning on option during play
		StartTimerGameTime(GameTimerInterval_Sustenance, GameTimerID_Sustenance)

	elseif aiTimerID == GameTimerID_Disease
		trace(self, "OnTimerGameTime() aiTimerID: = GameTimerID_Disease" )
		if IsGlobalTrue(HC_Rule_DiseaseEffects)
			HandleDiseaseTimer()
		endif
		;this needs to live outside the if check, in case we allow turning on option during play
		StartTimerGameTime(GameTimerInterval_Disease, GameTimerID_Disease)

	endif
EndEvent


bool Function Trace(ScriptObject CallingObject, string asTextToPrint, int aiSeverity = 0) debugOnly
	;we are sending callingObject so we can in the future route traces to different logs based on who is calling the function
	string logName = "Hardcore"
	debug.OpenUserLog(logName) 
	RETURN debug.TraceUser(logName, CallingObject + ": " + asTextToPrint, aiSeverity)
	
EndFunction

;NOW HAPPENS IN CODE
;NOT IN CURRENT BRANCH YET, THIS WILL NEED TO BE PULLED BACK OUT
Function SetHardcoreMode(bool HardcoreModeOn = true)
	int i = 0
	while (i < HC_Rules.GetSize())
		SetGlobal(HC_Rules.GetAt(i) as GlobalVariable, HardcoreModeOn)

		if HardcoreModeOn == false
			;otherstuff would need to go here, like turning off status effects, etc.
		endif

		i += 1
	endwhile
EndFunction

;convenience function, also helps enforce the values
bool function IsGlobalTrue(globalvariable GlobalToCheck)
	
	bool val = GlobalToCheck.GetValue() 

	if val == iGlobalTrue
		RETURN true
	elseif  val == iGlobalFalse
		RETURN false
	else
		;ERROR
		Game.Warning(self + "IsGlobalTrue() found unrecognized value in " + GlobalToCheck + ": " + GlobalToCheck.GetValue())
		RETURN false
	endif

EndFunction

;convenience function, also helps enforce the values
Function SetGlobal(globalvariable GlobalToSet, bool ValueToSet)
	if ValueToSet
		GlobalToSet.SetValue(iGlobalTrue)
	else 
		GlobalToSet.SetValue(iGlobalFalse)
	endif
EndFunction


;**************************************************************************************************
;*************************************    BALANCE CHANGES   ****************************************
;**************************************************************************************************

Group Balance
	globalvariable Property HC_Rule_ScaleDamage const auto mandatory  ;used by perk on Player scale damage based on actor valies
EndGroup


;**************************************************************************************************
;**************************************     FAST TRAVEL     ***************************************
;**************************************************************************************************

Group FastTravel
globalvariable Property HC_Rule_NoFastTravel const auto mandatory

EndGroup

;/ -- NOW IN CODE
InputEnableLayer FastTravelEnableLayer

;This may be handled by code, for the meantime this is a convenient hook for toggling fast travel allowance
;Registered for in OnInit()
Event OnMenuOpenCloseEvent(string asMenuName, bool abOpening)
	if asMenuName == "PipboyMenu" && abOpening

		if IsGlobalTrue(HC_Rule_NoFastTravel)
			if FastTravelEnableLayer == None
				FastTravelEnableLayer = InputEnableLayer.Create()
			endif

			if FastTravelEnableLayer.IsFastTravelEnabled()
				FastTravelEnableLayer.EnableFastTravel(false)
			endif
		elseif fastTravelEnableLayer
			FastTravelEnableLayer.delete() 
		endif
	endif
EndEvent
/;

;**************************************************************************************************
;**************************************      ADRENALINE     ***************************************
;**************************************************************************************************

Group Adrenaline
globalvariable Property HC_Rule_AdrenalineOn const auto mandatory
ActorValue Property HC_Adrenaline const auto mandatory
perk Property HC_AdrenalinePerk const auto mandatory
EndGroup

int killsForAdrenalinePerkLevel = 5 const ;how many kills before you gain an adrenaline perk level
int maxAdrenalinePerkLevel = 10 const
int AdrenalinePerkLevel ;there's no way to get the perk level, so we store it here manually


;Registered for in OnInit()
Event Actor.OnKill(Actor akSender, Actor akVictim)
	if akSender == PlayerRef ;registered only for player, but doesn't hurt to check
		trace(self, "Actor.OnKill() player killed: " + akVictim)

		;set adrenaline
		ModAdrenaline(1)
	endif

EndEvent

bool adrenalineLock
Function ModAdrenaline(int amountToMod)

	while adrenalineLock
		Utility.wait(0.01)
	endwhile

	adrenalineLock = true

	int currentValue = playerRef.GetValue(HC_Adrenaline) as int
	int newValue = currentValue + amountToMod

	trace(self, "ModAdrenaline() currentValue: " + currentValue + ", amountToMod: " + amountToMod + ", newValue: " + newValue)

	int maxAdrenaline = killsForAdrenalinePerkLevel * maxAdrenalinePerkLevel

	if newValue < 0
		trace(self, "ModAdrenaline() clamping NewValue to 0")
		newValue = 0
	elseif newValue > maxAdrenaline
		trace(self, "ModAdrenaline() clamping NewValue to maxAdrenaline: " + maxAdrenaline)
		newValue = maxAdrenaline
	endif
	
	playerRef.SetValue(HC_Adrenaline, newValue)

	;add perk rank if needed
	_UpdateAdrenalinePerk()

	adrenalineLock = false
EndFunction

Function _UpdateAdrenalinePerk()
	int currentAdrenaline = playerRef.GetValue(HC_Adrenaline) as int
	int perkLevelShouldBe = currentAdrenaline / killsForAdrenalinePerkLevel

	;clamp
	if perkLevelShouldBe > maxAdrenalinePerkLevel
		perkLevelShouldBe = maxAdrenalinePerkLevel
	endif

	trace(self, "_UpdateAdrenalinePerk() currentAdrenaline: " + currentAdrenaline + " perkLevelShouldBe: " + perkLevelShouldBe)

	;remove all ranks, then add them back
	playerRef.removePerk(HC_AdrenalinePerk)
	int i = perkLevelShouldBe
	while (i > 0)
		playerRef.AddPerk(HC_AdrenalinePerk)	
		i -= 1
	endwhile



EndFunction


;**************************************************************************************************
;**************************************        COMMON EFFECTS       ********************************
;**************************************************************************************************

Struct Effect
	Potion EffectPotion
{What potion to apply when gaining this effect - for all negative effects so they show up in stats effects list in pipboy}
	GlobalVariable GlobalEnum 
{this global's value is the enum for this effect}
	Message MessageToDisplay
{What message do we display when gaining this effect}
	Message MessageToDisplayAfterAwaking
{What message do we display when awaking with this effect}
	float DiseaseChanceMult = 0.5
{The multiplier on getting disease if player have this effect}

EndStruct

message EffectMessageToShow

Function ApplyEffect(GlobalVariable RuleGlobal, Effect[] EffectsArray, ActorValue EffectActorValue, bool DispellRestedSpells = false, bool IncrementEffect = false, bool DisplayAfterAwakingMessage = false)
	
	if IsGlobalTrue(RuleGlobal) == false
		;** This ASSUMES we don't toggle off and on in the same play through... if so, this could cause you to permamently be in the effected state)
		;BAIL OUT, not in hardcore mode
		RETURN
	endif

	;just incase there's base game spells running:
	if DispellRestedSpells
		playerRef.dispelSpell(WellRested)
		playerRef.dispelSpell(LoversEmbracePerkSpell)
	endif

	;give the player all the effects
	;they are potions he needs to equip, which will restart their duration
	;the effects on each potion are conditioned on the EffectActorValue actorvalue
	;I tried having each one dispel the effects on the other, similar to MS19MoleRatPoison and Cure... but that didn't work for some reason. Perhaps it'd only work on checking in?

	;restart all the  effects by equipping their potions on the player.
	int i = 0
	int EffectsLength = EffectsArray.length
	while (i < EffectsLength)
		potion potionToApply = EffectsArray[i].EffectPotion
	
		if potionToApply ;detrimental effects are potions
			PlayerRef.EquipItem(potionToApply, abSilent = true)
		endif

		i += 1
	endwhile


	;increment the  effect actor value
	int currentVal = PlayerRef.GetValue(EffectActorValue) as int
	int NewVal = currentVal
	if IncrementEffect
		newVal = currentVal + 1
	endif

	;don't exceed the array of effects
	if newVal < EffectsLength
		PlayerRef.SetValue(EffectActorValue, newVal)
	endif

	message PreviousEffectMessageToShow = EffectMessageToShow

	;display appropriate message
	if DisplayAfterAwakingMessage
		PreviousEffectMessageToShow = None ;always show messages upon waking
		EffectMessageToShow = EffectsArray[newVal].MessageToDisplayAfterAwaking
	else
		EffectMessageToShow = EffectsArray[newVal].MessageToDisplay
	endif

	if EffectMessageToShow != PreviousEffectMessageToShow
		EffectMessageToShow.show()
	endif

EndFunction


;**************************************************************************************************
;**************************************    SUSTENANCE EFFECTS  *******************************
;**************************************************************************************************

Group SustenanceEffects
	globalvariable Property HC_Rule_SustenanceEffects const auto mandatory

	ActorValue Property HC_HungerEffect const auto mandatory

	Effect[] Property HungerEffects const auto mandatory
{The order in this array, is the order they devolution after time passes since eating.}

	int Property FoodCostPerTick = 5 const auto
{in terms of caps value, how much food per tick is required to remain normal}

	keyword Property ObjectTypeFood const auto mandatory
{autofill}

	ActorValue Property HC_ThirstEffect const auto mandatory

	Effect[] Property ThirstEffects const auto mandatory
{The order in this array, is the order they devolution after time passes since eating.}

	int Property DrinkCostPerTick = 5 const auto
{in terms of caps value, how much food per tick is required to remain normal}

	float Property GamesHoursPerTick = 4.0 const auto
{how many game hours for a standard hunger/thirst check tick}

	float Property TickHoursCostPerCombat = 0.25 const auto 
{in terms of game hours, how much does each combat shave off the next tick time}

	keyword[] Property IncreasesThirstKeywords const auto mandatory
{keywords in here, removes points from Drink pool, making the player more thirsty}

	float Property IncreasesThirstCostMult = 0.5 const auto
{Default: 0.5; Multiplier on things that increase thirst's caps value}

	keyword[] Property QuenchesThirstKeywords const auto mandatory
{keywords in here, removes points from Drink pool, making the player more thirsty}

	float Property QuenchesThirstCostMult = 2.0 const auto
{Default: 2.0 ; Multiplier on things that quench thirst's caps value}

EndGroup

Group SustenanceEffectsGlobalEnums
	globalvariable Property HC_HE_Fed const auto
{autofill; global whose value represents this level}

	globalvariable Property HC_HE_Peckish const auto
{autofill; global whose value represents this level}

	globalvariable Property HC_HE_Hungry const auto
{autofill; global whose value represents this level}

	globalvariable Property HC_HE_Famished const auto
{autofill; global whose value represents this level}

	globalvariable Property HC_HE_Ravenous const auto
{autofill; global whose value represents this level}

	globalvariable Property HC_HE_Starving const auto
{autofill; global whose value represents this level}


	globalvariable Property HC_TE_Hydrated const auto
{autofill; global whose value represents this level}

	globalvariable Property HC_TE_Thirsty const auto
{autofill; global whose value represents this level}

	globalvariable Property HC_TE_Parched const auto
{autofill; global whose value represents this level}

	globalvariable Property HC_TE_MildlyDehydrated const auto
{autofill; global whose value represents this level}

	globalvariable Property HC_TE_Dehydrated const auto
{autofill; global whose value represents this level}

	globalvariable Property HC_TE_SeverelyDehyrdated const auto
{autofill; global whose value represents this level}


	globalvariable Property HC_SE_WellRestedORLoversEmbrace const auto
{autofill; global whose value represents this level}

	globalvariable Property HC_SE_Rested const auto
{autofill; global whose value represents this level}

	globalvariable Property HC_SE_Tired const auto
{autofill; global whose value represents this level}

	globalvariable Property HC_SE_Weary const auto
{autofill; global whose value represents this level}

	globalvariable Property HC_SE_Fatigued const auto
{autofill; global whose value represents this level}

	globalvariable Property HC_SE_Exhausted const auto
{autofill; global whose value represents this level}

	globalvariable Property HC_SE_Incapacitated const auto
{autofill; global whose value represents this level}

EndGroup

int FoodPool ; negative means a debt that needs to be paid or suffer effect
int DrinkPool ; negative means a debt that needs to be paid or suffer effect


float LastCombatDay                 ;store the last combat time in terms of game days passed
float MinDaysPerCombat = 0.1        ;how many gamedayspassed need to occur before we count a new combat

float NextSustenanceTickDay     ;expressed in GameDaysPassed, when is the next game day that a tick occurs - player getting in combats can shorten this.

Event Actor.OnCombatStateChanged(Actor akSender, Actor akTarget, int aeCombatState)
	float GameDaysPassed = Utility.GetCurrentGameTime() 

	if aeCombatState == 1 &&  GameDaysPassed < (LastCombatDay + MinDaysPerCombat)

		trace(self, "OnCombatStateChanged() shaving time off NextSustenanceTickDay. Was: " + NextSustenanceTickDay)

		;shave off time from tick day because player is exerting himself in combat
		NextSustenanceTickDay -=  (TickHoursCostPerCombat/24)

		trace(self, "OnCombatStateChanged() shaving time off NextSustenanceTickDay. Now: " + NextSustenanceTickDay)
		
		LastCombatDay = GameDaysPassed
	endif
EndEvent

Event Actor.OnItemEquipped(Actor akSender, Form akBaseObject, ObjectReference akReference)

	 if akBaseObject.HasKeyword(ObjectTypeFood)
	   int cost = akBaseObject.GetGoldValue()

	   trace(self, "OnItemEquipped() player eating food: " + akBaseObject  + ", cost: " + cost)

	   ModFoodPoolAndUpdateHungerEffects(cost)

	endif

	;this is not  ifelsed, because some food also cures thirst
	if IsThirstQuenching(akBaseObject)
		int cost = akBaseObject.GetGoldValue()

		trace(self, "OnItemEquipped() player quenching thirst: " + akBaseObject  + ", cost: " + cost)

		ModDrinkPoolAndUpdateThirstEffects(math.floor(cost * QuenchesThirstCostMult))

	elseif IsThirstIncreasing(akBaseObject)
		int cost = akBaseObject.GetGoldValue()

		trace(self, "OnItemEquipped() player quenching increasing thirst: " + akBaseObject  + ", cost: -" + cost)

		ModDrinkPoolAndUpdateThirstEffects(math.floor(-cost * IncreasesThirstCostMult))

	endif

EndEvent

bool Function IsThirstQuenching(form akBaseObject)

	bool returnVal = false
	if akBaseObject
		int i = 0
		while (returnVal == false && i < QuenchesThirstKeywords.length)
			returnVal = akBaseObject.HasKeyword(QuenchesThirstKeywords[i])
			i += 1
		endwhile
	endif

	return returnVal

EndFunction

bool Function IsThirstIncreasing(form akBaseObject)
	bool returnVal = false
	if akBaseObject
		int i = 0
		while (returnVal == false && i < IncreasesThirstKeywords.length)
			returnVal = akBaseObject.HasKeyword(IncreasesThirstKeywords[i])
			i += 1
		endwhile
	endif

	return returnVal	

EndFunction

function HandleSustenanceTimer()
	
	float GameDaysPassed = Utility.GetCurrentGameTime()
	
	;trace(self, "HandleSustenanceTimer() NextSustenanceTickDay: " + NextSustenanceTickDay)
	;trace(self, "HandleSustenanceTimer() GameDaysPassed: " + GameDaysPassed)
	
	;is this a tick threshold?
	if GameDaysPassed >= NextSustenanceTickDay

		trace(self, "HandleSustenanceTimer() GameDaysPassed >= NextSustenanceTickDay, process hunger")

		trace(self, "HandleSustenanceTimer() GameDaysPassed: " + GameDaysPassed + ", NextSustenanceTickDay: " + NextSustenanceTickDay + ", GamesHoursPerTick: " + GamesHoursPerTick)

		;get # of ticks since last time we handled the timer (in case player waits for a long time for example)
		int ticks =  math.floor( (GameDaysPassed - NextSustenanceTickDay) / (GamesHoursPerTick/24) ) 
		
		trace(self, "HandleSustenanceTimer() ticks: " + ticks)

		ModFoodPoolAndUpdateHungerEffects(-FoodCostPerTick * ticks)
		ModDrinkPoolAndUpdateThirstEffects(-DrinkCostPerTick * ticks)
	   
		;clamp food pool, overeating only helps for the current tick, it doesn't carry over
		if FoodPool > 0
			FoodPool = 0
		endif

		;clamp food pool, overeating only helps for the current tick, it doesn't carry over
		if DrinkPool > 0
			DrinkPool = 0
		endif

		;set up for the next tick
		NextSustenanceTickDay = GameDaysPassed + (GamesHoursPerTick/24)

	endif

endfunction

Function ModFoodPoolAndUpdateHungerEffects(int ModPoolAmount)

	FoodPool += ModPoolAmount     

	trace(self, "ModFoodPoolAndUpdateHungerEffects() ModPoolAmount: " + ModPoolAmount + ", new FoodPool: " + FoodPool)

	;based on food pool deficit, set hunger effect
		if foodPool <= -240
			;starving
			trace(self, "ModFoodPoolAndUpdateHungerEffects() player is Starving")
			PlayerRef.SetValue(HC_HungerEffect,  HC_HE_Starving.GetValue())

		elseif foodPool <= -120
			;ravenous
			trace(self, "ModFoodPoolAndUpdateHungerEffects() player is Ravenous")
			PlayerRef.SetValue(HC_HungerEffect, HC_HE_Ravenous.GetValue())

		elseif foodPool <= -60
			;famished
			trace(self, "ModFoodPoolAndUpdateHungerEffects() player is Famished")
			PlayerRef.SetValue(HC_HungerEffect, HC_HE_Famished.GetValue())

		elseif foodPool <= -30
			;hungry
			trace(self, "ModFoodPoolAndUpdateHungerEffects() player is Hungry")
			PlayerRef.SetValue(HC_HungerEffect, HC_HE_Hungry.GetValue())

		elseif foodPool <= -15
			;peckish
			trace(self, "ModFoodPoolAndUpdateHungerEffects() player is Peckish")
			PlayerRef.SetValue(HC_HungerEffect, HC_HE_Peckish.GetValue())

		else
			;fed
			trace(self, "ModFoodPoolAndUpdateHungerEffects() player is Fed")
			PlayerRef.SetValue(HC_HungerEffect, HC_HE_Fed.GetValue())

		endif


		ApplyEffect(HC_Rule_SustenanceEffects, HungerEffects, HC_HungerEffect, DispellRestedSpells = false, IncrementEffect = false)

EndFunction

Function ModDrinkPoolAndUpdateThirstEffects(int ModPoolAmount)

	DrinkPool += ModPoolAmount     

	trace(self, "ModDrinkPoolAndUpdateThirstEffects() ModPoolAmount: " + ModPoolAmount + ", new DrinkPool: " + DrinkPool)

	;based on Drink pool deficit, set effect
		if DrinkPool <= -150
			;Severly Dehydrated
			trace(self, "ModDrinkPoolAndUpdateThirstEffects() player is Severly Dehydrated")
			PlayerRef.SetValue(HC_ThirstEffect, HC_TE_SeverelyDehyrdated.GetValue())

		elseif DrinkPool <= -120
			;Dehydrated
			trace(self, "ModDrinkPoolAndUpdateThirstEffects() player is Dehydrated")
			PlayerRef.SetValue(HC_ThirstEffect, HC_TE_Dehydrated.GetValue())

		elseif DrinkPool <= -90
			;Mildly Dehyrdated
			trace(self, "ModDrinkPoolAndUpdateThirstEffects() player is Mildly Dehyrdated")
			PlayerRef.SetValue(HC_ThirstEffect, HC_TE_MildlyDehydrated.GetValue())

		elseif DrinkPool <= -60
			;Parched
			trace(self, "ModDrinkPoolAndUpdateThirstEffects() player is Parched")
			PlayerRef.SetValue(HC_ThirstEffect, HC_TE_Parched.GetValue())

		elseif DrinkPool <= -30
			;Thirsty
			trace(self, "ModDrinkPoolAndUpdateThirstEffects() player is Thirsty")
			PlayerRef.SetValue(HC_ThirstEffect, HC_TE_Thirsty.GetValue())

		else
			;Hydrated
			trace(self, "ModDrinkPoolAndUpdateThirstEffects() player is Hydrated")
			PlayerRef.SetValue(HC_ThirstEffect, HC_TE_Hydrated.GetValue())

		endif


		ApplyEffect(HC_Rule_SustenanceEffects, ThirstEffects, HC_ThirstEffect, DispellRestedSpells = false, IncrementEffect = false)

EndFunction



;**************************************************************************************************
;***********************************  		  DISEASE EFFECTS 		  *****************************
;**************************************************************************************************

Group Disease
	globalvariable Property HC_Rule_DiseaseEffects const auto mandatory

	Effect[] Property DiseaseEffects const auto mandatory
{Array of the various Disease Effects.}
;IMPORTANT NOTE: disease effects work a different than the others. They are simply effects we give the player that eventually time out.

	int Property DiseaseRiskPoolMinInput = 20 const auto
{Min % chance of disease in the pool... this is ultimately scaled by wellness state}

	int Property DiseaseRiskPoolMaxOutput = 90 const auto
{Max % chance of disease... no matter what, this is the max your chance of disease can go}

EndGroup

int DiseaseRiskPool ;positive number means higher chance of getting a disease

int lastDiseaseDieRoll = -1

Function CheckAndPossiblyApplyDisease(bool ShouldClearDiseaseRiskPool = false, int RiskyEventPoints = 0)
	trace(self, "CheckAndPossiblyApplyDisease() ShouldClearDiseaseRiskPool: " + ShouldClearDiseaseRiskPool )

	if IsGlobalTrue(HC_Rule_DiseaseEffects) == false
		;BAIL OUT, not in hardcore mode
		RETURN
	endif

	bool diseased = RollForDisease(RiskyEventPoints = RiskyEventPoints)

	if diseased
		;Randomly pick a disease and apply it
		int arrayLen = DiseaseEffects.length
		int dieRoll = utility.RandomInt(0, arrayLen - 1)

		;if same as last time, pick the higher one (they are arranged in array so that this increased chance these come in pairs will maybe make some thematic sense)
		if dieRoll == lastDiseaseDieRoll
			dieRoll += 1
		endif

		if dieRoll > arrayLen - 1
			dieRoll = 0
		endif

		lastDiseaseDieRoll = dieRoll

		effect effectToApply = DiseaseEffects[dieRoll]

		playerRef.EquipItem(effectToApply.EffectPotion, abSilent = true)
		effectToApply.MessageToDisplay.Show()
	endif
	

	if ShouldClearDiseaseRiskPool
		DiseaseRiskPool = 0
	endif
EndFunction

;returns true if roll was successful (ie should gain a disease)
bool Function RollForDisease(int RiskyEventPoints = 0)
	;RiskyEventPoints will be zero when called by on timer, and non-zero when called by risky event

	trace(self, "RollForDisease() RiskyEventPoints: " + RiskyEventPoints)

	;TO DO **** !!!!
	;roll chance for disease
	float WellnessMult = GetWellnessMult()
	float Chance = (DiseaseRiskPool + RiskyEventPoints) * WellnessMult

	int dieRoll = utility.RandomInt(1, 100) 


	trace(self, "RollForDisease() Chance: " + Chance)
	trace(self, "RollForDisease() dieRoll: " + dieRoll)

	if dieRoll <= Chance
		trace(self, "RollForDisease() dieRoll <= chance, returning: true")
		return true

	else
		trace(self, "RollForDisease() dieRoll > chance, returning: false")
		return false
	endif


EndFunction

;returns a multiplier to use when rolling for disease based on the players current hunger, thirst, and sleep effects
float Function GetWellnessMult()
	trace(self, "GetWellnessMult()")
	
	;figure out which effects by consulting player's current actor values
	effect HungerEffect = GetCurrentEffect(HC_HungerEffect, HungerEffects)
	effect ThirstEffect = GetCurrentEffect(HC_ThirstEffect, ThirstEffects)
	effect SleepEffect = GetCurrentEffect(HC_SleepEffect, SleepEffects)

	;average the effects' disease mults
	float HungerMult = HungerEffect.DiseaseChanceMult
	float ThirstMult = ThirstEffect.DiseaseChanceMult
	float SleepMult = SleepEffect.DiseaseChanceMult

	trace(self, "GetWellnessMult() HungerMult: " + HungerMult)
	trace(self, "GetWellnessMult() ThirstMult: " + ThirstMult)
	trace(self, "GetWellnessMult() SleepMult: " + SleepMult)

	float WellnessMult = (HungerMult + ThirstMult + SleepMult) / 3

	trace(self, "GetWellnessMult() returning: " + WellnessMult)

	return WellnessMult

EndFunction


effect Function GetCurrentEffect(ActorValue EffectActorValue, Effect[] EffectsArray)
	trace(self, "GetCurrentEffect() EffectActorValue: " + EffectActorValue + ", EffectsArray: " + EffectsArray)

	float val = PlayerRef.GetValue(EffectActorValue)

	;loop through the arrays, and find the struct who's globalenum value matches what is store in the player actor value
	int i = 0
	int max = EffectsArray.length
	while (i < max)
		effect currentEffect = EffectsArray[i]

		if currentEffect.GlobalEnum.GetValue() == val
			trace(self, "GetCurrentEffect() returning " + currentEffect)
			return currentEffect
		endif
		i += 1
	endwhile

	trace(self, "GetCurrentEffect() didn't find an effect", 2)

EndFunction


Function HandleDiseaseTimer()
	trace(self, "HandleDiseaseTimer()")
	CheckForDiseaseAndRestart()

EndFunction

Function CheckForDiseaseAndRestart()
	trace(self, "CheckForDiseaseAndRestart()")
	CheckAndPossiblyApplyDisease(ShouldClearDiseaseRiskPool = true)
	StartTimerGameTime(GameTimerInterval_Disease, GameTimerID_Disease)
EndFunction

Function HandleDiseaseRiskEvent(int RiskyEventPoints)
	trace(self, "HandleDiseaseRiskEvent() RiskyEventPoints: " + RiskyEventPoints)

	CheckAndPossiblyApplyDisease(ShouldClearDiseaseRiskPool = false, RiskyEventPoints = RiskyEventPoints)

	;store the RiskyEventPoints in the pool for handling again in the the future timer/sleep roll
	;we scale it by the wellness mult so that having taken care of yourself at the time of the risk event makes it less risky in the future
	DiseaseRiskPool += (RiskyEventPoints * GetWellnessMult()) as int

EndFunction


;**************************************************************************************************
;**************************************        SLEEP EFFECTS       ********************************
;**************************************************************************************************

;bedtype "enums"
int iBedType_NotApplicable = -1 const
int iBedType_SleepingBag = 0 const
int iBedType_Mattress = 1 const
int iBedType_Bed = 2 const



Group SleepEffects
	globalvariable Property HC_Rule_SleepEffects const auto mandatory

	ActorValue Property HC_SleepEffect const auto mandatory

	int Property MinHoursForCuringSleepEffects = 3 const auto
	{Need to at least sleep this long for sleep effects to be cured}

	keyword[] Property SleepingBagKeywords const auto mandatory
	{a bed with any of these keywords will be treated as a sleeping bag, wins over Mattress and Bed keywords}

	keyword[] Property MattressKeywords const auto mandatory
	{a bed with any of these keywords will be treated as a matress, wins over Bed keywords}

	keyword[] Property BedKeywords const auto mandatory
	{a bed with any of these keywords will be treated as a bed}

	Effect[] Property SleepEffects const auto mandatory
{The order in this array, is the order they devolution after time passes since sleeping.
***IMPORTANT REMINDER***
The first entry in this is a place hold for the Well Rested / Lover's Embrace effects, which are handled by pre-existing spells from base game not potions
}

	int Property IndexOfSleepEffectsForWellRestedORLoversEmbrace = 0 const auto
	{what index in SleepEffects represents WellRested}

	int Property HighestIndexOfSleepEffectsCuredBySleepingBag = 5 const auto
	{starting with this index in SleepEffects, you can cure this and higher effect while sleeping here}

	int Property HighestIndexOfSleepEffectsCuredByMattress = 3 const auto
	{starting with this index in SleepEffects, you can cure this and higher effect while sleeping here}

	int Property HighestIndexOfSleepEffectsCuredByBed = 2 const auto
	{starting with this index in SleepEffects, you can cure this and higher effect while sleeping here}

	message Property HC_SleepInterruptedMsg_Mattress const auto mandatory
	{autofill, message you get when you try to sleep longer than allowed}

	message Property HC_SleepInterruptedMsg_SleepingBag const auto mandatory
	{autofill, message you get when you try to sleep longer than allowed}

	FollowersScript Property Followers const auto mandatory
	{Autofill; needed for reimplementing lover's embrace}

	perk Property HC_WellRestedPerk const auto mandatory
	Message Property WellRestedMessage const Auto
	String Property WellRestedSWFname const Auto
	Sound Property UIPerkWellRested Const Auto Mandatory

	perk Property HC_LoversEmbracePerk const auto mandatory
	Message Property LoversEmbraceMessage const Auto
	String Property LoversEmbraceSWFname Const Auto 
	Sound Property UIPerkLoversEmbrace Const Auto Mandatory


	spell Property WellRested const auto mandatory
	{autofill, base game spell we'll need to dispell}

	spell Property LoversEmbracePerkSpell const auto mandatory
	{autofill, base game spell we'll need to dispell}

EndGroup


float SleepStartDay ;in terms of game days passed
float SleepStopDay ;in terms of game days passed

bool ProcessingSleep

Event OnPlayerSleepStart(float afSleepStartTime, float afDesiredSleepEndTime, ObjectReference akBed)
	
	ProcessingSleep = true

	SleepStartDay = afSleepStartTime

	;store existing health and condition values
	CacheValuesBeforeSleep()

	trace(self, "OnPlayerSleepStart() SleepStartDay: " + SleepStartDay)

	WakeUpPlayerBasedOnBedType(akBed, afSleepStartTime, afDesiredSleepEndTime)

EndEvent

Event OnPlayerSleepStop(bool abInterrupted, ObjectReference akBed)
	SleepStopDay = Utility.GetCurrentGameTime()
	float DaysSpentSleeping = SleepStopDay - SleepStartDay
	int GameHoursSpentSleeping = (DaysSpentSleeping * 24) as int

	trace(self, "OnPlayerSleepStart() SleepStopDay: " + SleepStopDay)   
	trace(self, "OnPlayerSleepStart() DaysSpentSleeping: " + DaysSpentSleeping)
	trace(self, "OnPlayerSleepStart() GameHoursSpentSleeping: " + GameHoursSpentSleeping)


	;Heal player's health and condition values based on how long he sleeped
	UpdateHealingAfterSleep(GameHoursSpentSleeping)


	;remove adrenaline based on number of hours
	int AdrenalineRanksToRemove

	if GameHoursSpentSleeping == 1
		AdrenalineRanksToRemove = 0
	elseif GameHoursSpentSleeping == 2
		AdrenalineRanksToRemove = 3
	elseif GameHoursSpentSleeping == 3
		AdrenalineRanksToRemove = 4
	elseif GameHoursSpentSleeping == 4
		AdrenalineRanksToRemove = 5
	elseif GameHoursSpentSleeping == 5
		AdrenalineRanksToRemove = 6
	elseif GameHoursSpentSleeping == 6
		AdrenalineRanksToRemove = 8
	else
		AdrenalineRanksToRemove = 10
	endif


	;REMOVE RANKS WORTH OF ADRENALINE
	int AdrelanineToRemove = AdrenalineRanksToRemove * killsForAdrenalinePerkLevel

	trace(self, "OnPlayerSleepStop() AdrelanineToRemove: " + AdrelanineToRemove )

	ModAdrenaline(-AdrelanineToRemove)

	;Update sleep effect based on bed and hours slept
	UpdateSleepEffectsAfterSleeping(GameHoursSpentSleeping, akBed)
	
	;roll for disease and restart timer
	CheckForDiseaseAndRestart()

	ProcessingSleep = false

	StartTimerGameTime(GameTimerInterval_SleepDeprivation, GameTimerID_SleepDeprivation) 
EndEvent

;*** WE NEED A "wakeup after X hours" FUNCTION, THIS ISN'T THE BEST WAY TO HANDLE THIS
;***Ideally we could get code to prevent you from dialing in more than you can sleep
Function WakeUpPlayerBasedOnBedType(ObjectReference BedRef, float SleepStartTime, float DesiredTime)
	float WakeDay = DesiredTime

	float MaxWakeDay
	message messageToShow

	if IsBed(BedRef)
		MaxWakeDay = SleepStartTime + 24.0 / 24.0 
	elseif IsSleepingBag(BedRef)
		MaxWakeDay = SleepStartTime + 3.0 / 24.0 
		messageToShow = HC_SleepInterruptedMsg_SleepingBag
	else ;treat everything else as mattress
		MaxWakeDay = SleepStartTime + 5.0 / 24.0
		messageToShow = HC_SleepInterruptedMsg_Mattress
	endif

	if MaxWakeDay >= WakeDay
		;no need to force you to wake up
		RETURN
	else
		WakeDay = MaxWakeDay
	endif

	float currentGameTime
	trace(self, "WakeUpPlayerBasedOnBedType() WakeDay: " + WakeDay + ", currentGameTime: " + currentGameTime)

	while (currentGameTime < WakeDay)
		utility.WaitMenuMode(0.5) ;sleeping happens pretty fast, need to catch it so we don't get more than an hour before we check again
		currentGameTime = utility.GetCurrentGameTime()

		trace(self, "WakeUpPlayerBasedOnBedType() WakeDay: " + WakeDay + ", currentGameTime: " + currentGameTime)
	endwhile

	;"wake up" the player
	PlayerRef.Moveto(PlayerRef) ;I worry a little bit about what this would do if the player was in a trigger, but in theory triggers should be able to handle this

	;explain why he woke up early
	messageToShow.show()

EndFunction

Function HandleSleepDeprivationTimer()
	
	;either of these perks only last 24 hours, so when the timer expires, just remove them
	playerRef.removePerk(HC_WellRestedPerk)
	playerRef.removePerk(HC_LoversEmbracePerk)

	ApplyEffect(HC_Rule_SleepEffects, SleepEffects, HC_SleepEffect, DispellRestedSpells = true, IncrementEffect = true)
EndFunction



Function UpdateSleepEffectsAfterSleeping(int GameHoursSpentSleeping, ObjectReference BedSleptIn)
	trace(self, "UpdateSleepEffectsAfterSleeping() GameHoursSpentSleeping: " + GameHoursSpentSleeping)

	int CurrentSleepEffect = PlayerRef.GetValue(HC_SleepEffect) as int

	int NewSleepEffect = 999

	bool SleepingInBed = IsBed(BedSleptIn)

	;first make sure we sleep long enough
	if GameHoursSpentSleeping >= MinHoursForCuringSleepEffects
		;roll back SleepEffects based on bed type
	
		if IsSleepingBag(BedSleptIn)
			NewSleepEffect = HighestIndexOfSleepEffectsCuredBySleepingBag
		elseif IsMattress(BedSleptIn)
			NewSleepEffect = HighestIndexOfSleepEffectsCuredByMattress
		elseif SleepingInBed
			NewSleepEffect = HighestIndexOfSleepEffectsCuredByBed
		else ;unexpected, consider it mattress
			NewSleepEffect = HighestIndexOfSleepEffectsCuredByMattress 
		endif

		NewSleepEffect -= 1 ;because you cure down to the previous SleepEffectslee
	
	endif

	;throw away well rested and lovers embrace, so you can't "top it off" by sleeping before it expires
	if CurrentSleepEffect == IndexOfSleepEffectsForWellRestedORLoversEmbrace
		CurrentSleepEffect += 1

		playerRef.removePerk(HC_WellRestedPerk)
		playerRef.removePerk(HC_LoversEmbracePerk)

	endif

	;if you sleep for 8 hours in a bed, you get well rested.
	if GameHoursSpentSleeping >= 8 && SleepingInBed
		trace(self, "UpdateSleepEffectsAfterSleeping() GameHoursSpentSleeping >= 8 && SleepingInBed, will give well rested or lovers embrace.")

		NewSleepEffect = IndexOfSleepEffectsForWellRestedORLoversEmbrace

		;apply the appropriate perk:
		if Followers.GetNearbyInfatuatedRomanticCompanion()
			playerRef.AddPerk(HC_LoversEmbracePerk)
			LoversEmbraceMessage.Show()
			Game.ShowPerkVaultBoyOnHUD(LoversEmbraceSwfName, UIPerkLoversEmbrace)

		else
			playerRef.AddPerk(HC_WellRestedPerk)
			WellRestedMessage.Show()
			Game.ShowPerkVaultBoyOnHUD(WellRestedSWFname, UIPerkWellRested)

		endif

	endif

	;only set it if it's better than the current effect
	if NewSleepEffect < CurrentSleepEffect
		trace(self, "UpdateSleepEffectsAfterSleeping() setting HC_SleepEffect to: " + NewSleepEffect)
		PlayerRef.SetValue(HC_SleepEffect, NewSleepEffect)
	else
		trace(self, "UpdateSleepEffectsAfterSleeping() setting HC_SleepEffect to: " + CurrentSleepEffect)
		PlayerRef.SetValue(HC_SleepEffect, CurrentSleepEffect)
	endif

	ApplyEffect(HC_Rule_SleepEffects, SleepEffects, HC_SleepEffect, DispellRestedSpells = true, DisplayAfterAwakingMessage = true)

EndFunction


bool Function IsSleepingBag(ObjectReference RefToCheck)
	return CommonArrayFunctions.CheckObjectAgainstKeywordArray(RefToCheck, SleepingBagKeywords)
EndFunction

bool Function IsMattress(ObjectReference RefToCheck)
	return CommonArrayFunctions.CheckObjectAgainstKeywordArray(RefToCheck, MattressKeywords)
EndFunction

bool Function IsBed(ObjectReference RefToCheck)
	return CommonArrayFunctions.CheckObjectAgainstKeywordArray(RefToCheck, BedKeywords)
EndFunction

;--------------------------------------------------------------------------------------------------
;------------------------------------    SLEEP EFFECTS -- HEALING    ------------------------------
;--------------------------------------------------------------------------------------------------
;Change how the auto heal works when sleeping. Also see OnPlayerSleepXXX events above.

Group SleepEffectsHealing
ActorValue Property Health const auto mandatory     ;Health
ActorValue Property EnduranceCondition const auto mandatory     ;TORSO
ActorValue Property LeftAttackCondition const auto mandatory    ;LEFT ARM   
ActorValue Property LeftMobilityCondition const auto mandatory  ;LEFT LEG
ActorValue Property PerceptionCondition const auto mandatory    ;HEAD
ActorValue Property RightAttackCondition const auto mandatory   ;RIGHT ARM
ActorValue Property RightMobilityCondition const auto mandatory ;RIGHT ARM
EndGroup


;Pre-sleep cached values
float HealthCache
float EnduranceConditionCache
float LeftAttackConditionCache
float LeftMobilityConditionCache
float PerceptionConditionCache
float RightAttackConditionCache
float RightMobilityConditionCache

Function CacheValuesBeforeSleep()
	HealthCache = PlayerRef.GetValue(Health)
	EnduranceConditionCache = PlayerRef.GetValue(EnduranceCondition)
	LeftAttackConditionCache = PlayerRef.GetValue(LeftAttackCondition)
	LeftMobilityConditionCache = PlayerRef.GetValue(LeftMobilityCondition)
	PerceptionConditionCache = PlayerRef.GetValue(PerceptionCondition)
	RightAttackConditionCache = PlayerRef.GetValue(RightAttackCondition)
	RightMobilityConditionCache = PlayerRef.GetValue(RightMobilityCondition)
EndFunction

Function UpdateHealingAfterSleep(int GameHoursSpentSleeping)

	;damage actorvalues back to cached values
	DamageValuesBackToCachedValues() ;*** BETTER HANDLED BY CODE?

	RestoreValueBasedOnHours(Health, GameHoursSpentSleeping)
	RestoreValueBasedOnHours(EnduranceCondition, GameHoursSpentSleeping)
	RestoreValueBasedOnHours(LeftAttackCondition, GameHoursSpentSleeping)
	RestoreValueBasedOnHours(LeftMobilityCondition, GameHoursSpentSleeping)
	RestoreValueBasedOnHours(PerceptionCondition, GameHoursSpentSleeping)
	RestoreValueBasedOnHours(RightAttackCondition, GameHoursSpentSleeping)
	RestoreValueBasedOnHours(RightMobilityCondition, GameHoursSpentSleeping)

EndFunction

;***THIS MIGHT BE BETTER HANDLED BY CODE SIMPLY NOT AUTOHEALING IN HARDCORE MODE
Function DamageValuesBackToCachedValues()
	DamageValueBackToCachedValue(Health, HealthCache)
	DamageValueBackToCachedValue(EnduranceCondition, EnduranceConditionCache)
	DamageValueBackToCachedValue(LeftAttackCondition, LeftAttackConditionCache)
	DamageValueBackToCachedValue(LeftMobilityCondition, LeftMobilityConditionCache)
	DamageValueBackToCachedValue(PerceptionCondition, PerceptionConditionCache)
	DamageValueBackToCachedValue(RightAttackCondition, RightAttackConditionCache)
	DamageValueBackToCachedValue(RightMobilityCondition, RightMobilityConditionCache)
EndFunction


Function DamageValueBackToCachedValue(ActorValue ActorValueToDamage, float CachedValue)
	float currentVal = PlayerRef.GetValue(ActorValueToDamage)
	float difference = currentVal - CachedValue
	
	trace(self, "DamageValueBackToCachedValue() ActorValueToDamage: " + ActorValueToDamage + ", currentVal:" + currentVal +  ", CachedValue: " + CachedValue +", difference: " + difference) 

	;this shouldn't happen, but just in case, bail out if current value is less than cached value (suggesting some kind of on going effect that's not healed by sleeping)
	if difference <= 0 
		trace(self, "DamageValueBackToCachedValue() BAILING OUT because difference is negative or zero. difference: " + difference)
		RETURN
	endif

	trace(self, "DamageValueBackToCachedValue() damaging ActorValue " + ActorValueToDamage + ", " + difference)
	PlayerRef.DamageValue(ActorValueToDamage, difference)
EndFunction

Function RestoreValueBasedOnHours(ActorValue ActorValueToRestore, int GameHoursSpentSleeping)
	trace(self, "RestoreValueBasedOnHours() ActorValueToRestore: " + ActorValueToRestore + ", GameHoursSpentSleeping: " + GameHoursSpentSleeping)


	float valueToRestore = PlayerRef.GetValue(ActorValueToRestore)
	
	;restore a % of max total based on hours slept
	if GameHoursSpentSleeping == 1
		valueToRestore *= 0.00
	elseif GameHoursSpentSleeping == 2
		valueToRestore *= 0.15
	elseif GameHoursSpentSleeping == 3
		valueToRestore *= 0.25
	elseif GameHoursSpentSleeping == 4
		valueToRestore *= 0.45
	elseif GameHoursSpentSleeping == 5
		valueToRestore *= 0.75
	else 
		valueToRestore *= 1 
	endif

	trace(self, "RestoreValueBasedOnHours() valueToRestore: " + valueToRestore)
	PlayerRef.RestoreValue(ActorValueToRestore, valueToRestore)

EndFunction

;**************************************************************************************************
;***********************************  ENCUMBRANCE AND LIMB CONDITION  *****************************
;**************************************************************************************************

Group EncumbranceAndLimbCondition
	
	globalvariable Property HC_Rule_NoLimbConditionHeal const auto mandatory
	globalvariable Property HC_Rule_DamageWhenEncumbered const auto mandatory

	potion Property HC_EncumbranceEffect_OverEncumbered const auto mandatory

EndGroup

Function HandleEncumbranceTimer()
	;make the player drink the potion that turns on the encumberance effect (similar to how sleep effects work)
	;spell effects in potion are conditioned on player being encumbered 
	PlayerRef.EquipItem(HC_EncumbranceEffect_OverEncumbered, abSilent = true)
	StartTimerGameTime(GameTimerInterval_Encumbrance, GameTimerID_Encumbrance)
EndFunction 


